@@ -38,6 +38,7 @@ int _kbhit() {
 }
 #endif
 
+#include "resip/stack/InteropHelper.hxx"
 #include "recon/UserAgent.hxx"
 #include "recon/ReconSubsystem.hxx"
 #include <recon/SipXHelper.hxx>
@@ -49,6 +50,7 @@ int _kbhit() {
 #include "MyMessageDecorator.hxx"
 #include "MyConversationManager.hxx"
 #include "B2BCallManager.hxx"
+#include "RegistrationForwarder.hxx"
 
 #include <rutil/Log.hxx>
 #include <rutil/Logger.hxx>
@@ -56,8 +58,12 @@ int _kbhit() {
 #include <rutil/BaseException.hxx>
 #include <rutil/WinLeakCheck.hxx>
 
+#include <resip/stack/HEPSipMessageLoggingHandler.hxx>
+#include <reflow/HEPRTCPEventLoggingHandler.hxx>
+
 using namespace recon;
 using namespace resip;
+using namespace flowmanager;
 using namespace std;
 
 #define RESIPROCATE_SUBSYSTEM ReconSubsystem::RECON
@@ -83,28 +89,6 @@ signalHandler(int signo)
    finished = true;
 }
 
-class MyUserAgent : public UserAgent
-{
-public:
-   MyUserAgent(ConversationManager* conversationManager, SharedPtr<UserAgentMasterProfile> profile) :
-      UserAgent(conversationManager, profile) {}
-
-   virtual void onApplicationTimer(unsigned int id, unsigned int durationMs, unsigned int seq)
-   {
-      InfoLog(<< "onApplicationTimeout: id=" << id << " dur=" << durationMs << " seq=" << seq);
-   }
-
-   virtual void onSubscriptionTerminated(SubscriptionHandle handle, unsigned int statusCode)
-   {
-      InfoLog(<< "onSubscriptionTerminated: handle=" << handle << " statusCode=" << statusCode);
-   }
-
-   virtual void onSubscriptionNotify(SubscriptionHandle handle, Data& notifyData)
-   {
-      InfoLog(<< "onSubscriptionNotify: handle=" << handle << " data=" << endl << notifyData);
-   }
-};
-
 int main(int argc, char** argv)
 {
    ReConServerProcess proc;
@@ -835,21 +819,31 @@ ReConServerProcess::main (int argc, char** argv)
    ConversationProfile::SecureMediaMode secureMediaMode = reConServerConfig.getConfigSecureMediaMode("SecureMediaMode", ConversationProfile::NoSecureMedia);
    bool secureMediaRequired = reConServerConfig.isSecureMediaModeRequired();
    ConversationProfile::NatTraversalMode natTraversalMode = reConServerConfig.getConfigNatTraversalMode("NatTraversalMode", ConversationProfile::NoNatTraversal);
+   bool forceCOMedia = reConServerConfig.getConfigBool("ForceCOMedia", true);
    Data natTraversalServerHostname = reConServerConfig.getConfigData("NatTraversalServerHostname", "", true);
    unsigned short natTraversalServerPort = reConServerConfig.getConfigUnsignedShort("NatTraversalServerPort", 3478);
    Data stunUsername = reConServerConfig.getConfigData("StunUsername", "", true);
    Data stunPassword = reConServerConfig.getConfigData("StunPassword", "", true);
+   bool addViaRport = reConServerConfig.getConfigBool("AddViaRport", true);
    unsigned short tcpPort = reConServerConfig.getConfigUnsignedShort("TCPPort", 5062);
    unsigned short udpPort = reConServerConfig.getConfigUnsignedShort("UDPPort", 5062);
    unsigned short tlsPort = reConServerConfig.getConfigUnsignedShort("TLSPort", 5063);
    unsigned short mediaPortStart = reConServerConfig.getConfigUnsignedShort("MediaPortStart", 17384);
    Data tlsDomain = reConServerConfig.getConfigData("TLSDomain", DnsUtil::getLocalHostName(), true);
    NameAddr outboundProxy = reConServerConfig.getConfigNameAddr("OutboundProxyUri", NameAddr(), true);
+#ifdef PACKAGE_VERSION
+   Data serverText = reConServerConfig.getConfigData("ServerText", "reConServer " PACKAGE_VERSION);
+#else
+   Data serverText = reConServerConfig.getConfigData("ServerText", "reConServer");
+#endif
    uri = reConServerConfig.getConfigNameAddr("SIPUri", uri, true);
    Data loggingType = reConServerConfig.getConfigData("LoggingType", "cout", true);
    Data loggingLevel = reConServerConfig.getConfigData("LoggingLevel", "INFO", true);
    Data loggingFilename = reConServerConfig.getConfigData("LogFilename", "reConServer.log", true);
    unsigned int loggingFileMaxLineCount = reConServerConfig.getConfigUnsignedLong("LogFileMaxLines", 50000);
+   Data captureHost = reConServerConfig.getConfigData("CaptureHost", "");
+   int capturePort = reConServerConfig.getConfigInt("CapturePort", 9060);
+   int captureAgentID = reConServerConfig.getConfigInt("CaptureAgentID", 2002);
    bool localAudioEnabled = reConServerConfig.getConfigBool("EnableLocalAudio", !daemonize); // Defaults to false for daemon process
    Data runAsUser = reConServerConfig.getConfigData("RunAsUser", "", true);
    Data runAsGroup = reConServerConfig.getConfigData("RunAsGroup", "", true);
@@ -862,25 +856,29 @@ ReConServerProcess::main (int argc, char** argv)
    ReConServerConfig::Application application = reConServerConfig.getConfigApplication("Application", ReConServerConfig::None);
 
 
+   // build a list of codecs in priority order
+   // Used by ConversationManager::buildSessionCapabilities(...) to create
+   // our local SDP
    std::vector<unsigned int> _codecIds;
-   _codecIds.push_back(SdpCodec::SDP_CODEC_PCMU);           // 0 - pcmu
-   _codecIds.push_back(SdpCodec::SDP_CODEC_PCMA);           // 8 - pcma
-   _codecIds.push_back(SdpCodec::SDP_CODEC_SPEEX);          // 96 - speex NB 8,000bps
-   _codecIds.push_back(SdpCodec::SDP_CODEC_SPEEX_15);       // 98 - speex NB 15,000bps
-   _codecIds.push_back(SdpCodec::SDP_CODEC_SPEEX_24);       // 99 - speex NB 24,600bps
-   _codecIds.push_back(SdpCodec::SDP_CODEC_L16_44100_MONO); // PCM 16 bit/sample 44100 samples/sec.
-   _codecIds.push_back(SdpCodec::SDP_CODEC_ILBC);           // 108 - iLBC
-   _codecIds.push_back(SdpCodec::SDP_CODEC_ILBC_20MS);      // 109 - Internet Low Bit Rate Codec, 20ms (RFC3951)
-   _codecIds.push_back(SdpCodec::SDP_CODEC_SPEEX_5);        // 97 - speex NB 5,950bps
-   _codecIds.push_back(SdpCodec::SDP_CODEC_GSM);            // 3 - GSM
-   if(enableG722)
-   {
-      _codecIds.push_back(SdpCodec::SDP_CODEC_G722);        // 9 - G.722
-   }
    if(enableOpus)
    {
       _codecIds.push_back(SdpCodec::SDP_CODEC_OPUS);        // Opus
    }
+   if(enableG722)
+   {
+      _codecIds.push_back(SdpCodec::SDP_CODEC_G722);        // 9 - G.722
+   }
+   _codecIds.push_back(SdpCodec::SDP_CODEC_ILBC);           // 108 - iLBC
+   _codecIds.push_back(SdpCodec::SDP_CODEC_ILBC_20MS);      // 109 - Internet Low Bit Rate Codec, 20ms (RFC3951)
+   _codecIds.push_back(SdpCodec::SDP_CODEC_SPEEX_24);       // 99 - speex NB 24,600bps
+   _codecIds.push_back(SdpCodec::SDP_CODEC_SPEEX_15);       // 98 - speex NB 15,000bps
+   _codecIds.push_back(SdpCodec::SDP_CODEC_SPEEX);          // 96 - speex NB 8,000bps
+   _codecIds.push_back(SdpCodec::SDP_CODEC_SPEEX_5);        // 97 - speex NB 5,950bps
+   _codecIds.push_back(SdpCodec::SDP_CODEC_GSM);            // 3 - GSM
+   //_codecIds.push_back(SdpCodec::SDP_CODEC_L16_44100_MONO); // PCM 16 bit/sample 44100 samples/sec.
+   _codecIds.push_back(SdpCodec::SDP_CODEC_PCMU);           // 0 - pcmu
+   _codecIds.push_back(SdpCodec::SDP_CODEC_PCMA);           // 8 - pcma
+   _codecIds.push_back(SdpCodec::SDP_CODEC_G729);           // 18 - G.729
    _codecIds.push_back(SdpCodec::SDP_CODEC_TONES);          // 110 - telephone-event
    unsigned int *codecIds = &_codecIds[0];
    unsigned int numCodecIds = _codecIds.size();
@@ -940,22 +938,132 @@ ReConServerProcess::main (int argc, char** argv)
 
    SharedPtr<UserAgentMasterProfile> profile(new UserAgentMasterProfile);
 
+   if(!captureHost.empty())
+   {
+      SharedPtr<HepAgent> agent(new HepAgent(captureHost, capturePort, captureAgentID));
+      profile->setTransportSipMessageLoggingHandler(SharedPtr<HEPSipMessageLoggingHandler>(new HEPSipMessageLoggingHandler(agent)));
+      profile->setRTCPEventLoggingHandler(SharedPtr<HEPRTCPEventLoggingHandler>(new HEPRTCPEventLoggingHandler(agent)));
+   }
+
    // Add transports
-   if(udpPort)
+   try
    {
-      profile->addTransport(UDP, udpPort, V4, address);
-   }
-   if(tcpPort)
-   {
-      profile->addTransport(TCP, tcpPort, V4, address);
-   }
+      bool useEmailAsSIP = reConServerConfig.getConfigBool("TLSUseEmailAsSIP", false);
+
+      // Check if advanced transport settings are provided
+      ConfigParse::NestedConfigMap m = reConServerConfig.getConfigNested("Transport");
+      DebugLog(<<"Found " << m.size() << " interface(s) defined in the advanced format");
+      if(!m.empty())
+      {
+         // Sample config file format for advanced transport settings
+         // Transport1Interface = 192.168.1.106:5061
+         // Transport1Type = TLS
+         // Transport1TlsDomain = sipdomain.com
+         // Transport1TlsCertificate = /etc/ssl/crt/sipdomain.com.pem
+         // Transport1TlsPrivateKey = /etc/ssl/private/sipdomain.com.pem
+         // Transport1TlsPrivateKeyPassPhrase = <pwd>
+         // Transport1TlsClientVerification = None
+         // Transport1RcvBufLen = 2000
+
+         const char *anchor;
+         for(ConfigParse::NestedConfigMap::iterator it = m.begin();
+            it != m.end();
+            it++)
+         {
+            int idx = it->first;
+            SipConfigParse tc(it->second);
+            Data transportPrefix = "Transport" + idx;
+            DebugLog(<< "checking values for transport: " << idx);
+            Data interfaceSettings = tc.getConfigData("Interface", Data::Empty, true);
+
+            // Parse out interface settings
+            ParseBuffer pb(interfaceSettings);
+            anchor = pb.position();
+            pb.skipToEnd();
+            pb.skipBackToChar(':');  // For IPv6 the last : should be the port
+            pb.skipBackChar();
+            if(!pb.eof())
+            {
+               Data ipAddr;
+               Data portData;
+               pb.data(ipAddr, anchor);
+               pb.skipChar();
+               anchor = pb.position();
+               pb.skipToEnd();
+               pb.data(portData, anchor);
+               if(!DnsUtil::isIpAddress(ipAddr))
+               {
+                  CritLog(<< "Malformed IP-address found in " << transportPrefix << "Interface setting: " << ipAddr);
+               }
+               int port = portData.convertInt();
+               if(port == 0)
+               {
+                  CritLog(<< "Invalid port found in " << transportPrefix << " setting: " << port);
+               }
+               TransportType tt = Tuple::toTransport(tc.getConfigData("Type", "UDP"));
+               if(tt == UNKNOWN_TRANSPORT)
+               {
+                  CritLog(<< "Unknown transport type found in " << transportPrefix << "Type setting: " << tc.getConfigData("Type", "UDP"));
+               }
+               Data tlsDomain = tc.getConfigData("TlsDomain", Data::Empty);
+               Data tlsCertificate = tc.getConfigData("TlsCertificate", Data::Empty);
+               Data tlsPrivateKey = tc.getConfigData("TlsPrivateKey", Data::Empty);
+               Data tlsPrivateKeyPassPhrase = tc.getConfigData("TlsPrivateKeyPassPhrase", Data::Empty);
+               SecurityTypes::TlsClientVerificationMode cvm = tc.getConfigClientVerificationMode("TlsClientVerification", SecurityTypes::None);
+               SecurityTypes::SSLType sslType = SecurityTypes::NoSSL;
 #ifdef USE_SSL
-   if(tlsPort)
-   {
-      profile->addTransport(TLS, tlsPort, V4, address, tlsDomain);
-   }
+               sslType = tc.getConfigSSLType("TlsConnectionMethod", SecurityTypes::SSLv23);
 #endif
 
+               int rcvBufLen = tc.getConfigInt("RcvBufLen", 0);
+
+               profile->addTransport(tt,
+                                 port,
+                                 DnsUtil::isIpV6Address(ipAddr) ? V6 : V4,
+                                 StunEnabled,
+                                 ipAddr,       // interface to bind to
+                                 tlsDomain,
+                                 tlsPrivateKeyPassPhrase,  // private key passphrase
+                                 sslType, // sslType
+                                 0,            // transport flags
+                                 tlsCertificate, tlsPrivateKey,
+                                 cvm,          // tls client verification mode
+                                 useEmailAsSIP, rcvBufLen);
+
+            }
+            else
+            {
+               CritLog(<< "Port not specified in " << transportPrefix << " setting: expected format is <IPAddress>:<Port>");
+               return false;
+            }
+         }
+      }
+      else
+      {
+         DebugLog(<<"Using legacy transport configuration");
+         if(udpPort)
+         {
+            profile->addTransport(UDP, udpPort, V4, StunEnabled, address, Data::Empty, Data::Empty, SecurityTypes::SSLv23, 0, Data::Empty, Data::Empty, SecurityTypes::None, useEmailAsSIP);
+         }
+         if(tcpPort)
+         {
+            profile->addTransport(TCP, tcpPort, V4, StunEnabled, address, Data::Empty, Data::Empty, SecurityTypes::SSLv23, 0, Data::Empty, Data::Empty, SecurityTypes::None, useEmailAsSIP);
+         }
+#ifdef USE_SSL
+         if(tlsPort)
+         {
+            profile->addTransport(TLS, tlsPort, V4, StunEnabled, address, tlsDomain, Data::Empty, SecurityTypes::SSLv23, 0, Data::Empty, Data::Empty, SecurityTypes::None, useEmailAsSIP);
+         }
+#endif
+      }
+   }
+   catch (BaseException& e)
+   {
+      std::cerr << "Likely a port is already in use" << endl;
+      InfoLog (<< "Caught: " << e);
+      return false;
+   }
+
    // The following settings are used to avoid a kernel panic seen on an ARM embedded platform.
    // The kernel panic happens when either binding a udp socket to port 0 (OS selected),
    // or calling connect without first binding to a specific port.  There is code in the
@@ -1079,7 +1187,7 @@ ReConServerProcess::main (int argc, char** argv)
       profile->setOutboundProxy(outboundProxy.uri());
    }
 
-   profile->setUserAgent("ConversationManager/reConServer");
+   profile->setUserAgent(serverText);
    profile->rtpPortRangeMin() = mediaPortStart;
    profile->rtpPortRangeMax() = mediaPortStart + 101; // Allows 100 media streams
 
@@ -1120,12 +1228,6 @@ ReConServerProcess::main (int argc, char** argv)
 
    // Build Codecs and media offering
    SdpContents::Session::Medium medium("audio", port, 1, "RTP/AVP");
-   if(enableG722)
-   {
-      SdpContents::Session::Codec g722codec("G722", 8000);
-      g722codec.payloadType() = 9;  /* RFC3551 */ ;
-      medium.addCodec(g722codec);
-   }
    if(enableOpus)
    {
       // Note: the other constructors (e.g. g722 above) are
@@ -1136,19 +1238,28 @@ ReConServerProcess::main (int argc, char** argv)
       opuscodec.encodingParameters() = Data("2");
       medium.addCodec(opuscodec);
    }
+   if(enableG722)
+   {
+      SdpContents::Session::Codec g722codec("G722", 8000);
+      g722codec.payloadType() = 9;  /* RFC3551 */ ;
+      medium.addCodec(g722codec);
+   }
+   SdpContents::Session::Codec speexCodec("SPEEX", 8000);
+   speexCodec.payloadType() = 110;
+   speexCodec.parameters() = Data("mode=3");
+   medium.addCodec(speexCodec);
+   SdpContents::Session::Codec gsmCodec("GSM", 8000);
+   gsmCodec.payloadType() = 3;  /* RFC3551 */ ;
+   medium.addCodec(gsmCodec);
    SdpContents::Session::Codec g711ucodec("PCMU", 8000);
    g711ucodec.payloadType() = 0;  /* RFC3551 */ ;
    medium.addCodec(g711ucodec);
    SdpContents::Session::Codec g711acodec("PCMA", 8000);
    g711acodec.payloadType() = 8;  /* RFC3551 */ ;
    medium.addCodec(g711acodec);
-   SdpContents::Session::Codec speexCodec("SPEEX", 8000);
-   speexCodec.payloadType() = 110;  
-   speexCodec.parameters() = Data("mode=3");
-   medium.addCodec(speexCodec);
-   SdpContents::Session::Codec gsmCodec("GSM", 8000);
-   gsmCodec.payloadType() = 3;  /* RFC3551 */ ;
-   medium.addCodec(gsmCodec);
+   SdpContents::Session::Codec g729codec("G729", 8000);
+   g729codec.payloadType() = 18;  /* RFC3551 */ ;
+   medium.addCodec(g729codec);
    medium.addAttribute("ptime", Data(20));  // 20 ms of speech per frame (note G711 has 10ms samples, so this is 2 samples per frame)
    medium.addAttribute("sendrecv");
 
@@ -1163,10 +1274,13 @@ ReConServerProcess::main (int argc, char** argv)
 
    // Setup NatTraversal Settings
    conversationProfile->natTraversalMode() = natTraversalMode;
+   conversationProfile->forceCOMedia() = forceCOMedia;
    conversationProfile->natTraversalServerHostname() = natTraversalServerHostname;
    conversationProfile->natTraversalServerPort() = natTraversalServerPort;
    conversationProfile->stunUsername() = stunUsername;
    conversationProfile->stunPassword() = stunPassword;
+   InteropHelper::setRportEnabled(addViaRport);
+   conversationProfile->setRportEnabled(addViaRport);
 
    // Secure Media Settings
    conversationProfile->secureMediaMode() = secureMediaMode;
@@ -1177,28 +1291,47 @@ ReConServerProcess::main (int argc, char** argv)
    // Create ConverationManager and UserAgent
    //////////////////////////////////////////////////////////////////////////////
    {
-      std::auto_ptr<MyConversationManager> myConversationManager;
+      std::auto_ptr<MyConversationManager> conversationManager;
       switch(application)
       {
          case ReConServerConfig::None:
-            myConversationManager.reset(new MyConversationManager(localAudioEnabled, mediaInterfaceMode, defaultSampleRate, maximumSampleRate, autoAnswerEnabled));
+            conversationManager.reset(new MyConversationManager(localAudioEnabled, mediaInterfaceMode, defaultSampleRate, maximumSampleRate, autoAnswerEnabled));
             break;
          case ReConServerConfig::B2BUA:
-            myConversationManager.reset(new B2BCallManager(mediaInterfaceMode, defaultSampleRate, maximumSampleRate, reConServerConfig));
+            conversationManager.reset(new B2BCallManager(mediaInterfaceMode, defaultSampleRate, maximumSampleRate, reConServerConfig));
             break;
          default:
             assert(0);
       }
-      MyUserAgent ua(myConversationManager.get(), profile);
-      myConversationManager->buildSessionCapabilities(address, numCodecIds, codecIds, conversationProfile->sessionCaps());
+      MyUserAgent ua(reConServerConfig, conversationManager.get(), profile);
+      conversationManager->buildSessionCapabilities(address, numCodecIds, codecIds, conversationProfile->sessionCaps());
       ua.addConversationProfile(conversationProfile);
 
+      if(application == ReConServerConfig::B2BUA)
+      {
+         Data internalMediaAddress;
+         reConServerConfig.getConfigValue("B2BUAInternalMediaAddress", internalMediaAddress);
+         if(!internalMediaAddress.empty())
+         {
+            SharedPtr<ConversationProfile> internalProfile(new ConversationProfile(conversationProfile));
+            internalProfile->secureMediaMode() = reConServerConfig.getConfigSecureMediaMode("B2BUAInternalSecureMediaMode", secureMediaMode);
+            internalProfile->setDefaultFrom(uri);
+            internalProfile->setDigestCredential(uri.uri().host(), uri.uri().user(), password);
+            conversationManager->buildSessionCapabilities(internalMediaAddress, numCodecIds, codecIds, internalProfile->sessionCaps());
+            ua.addConversationProfile(internalProfile, false);
+         }
+         else
+         {
+            WarningLog(<<"B2BUAInternalMediaAddress not specified, using same media address for internal and external zones");
+         }
+      }
+
       //////////////////////////////////////////////////////////////////////////////
       // Startup and run...
       //////////////////////////////////////////////////////////////////////////////
 
       ua.startup();
-      myConversationManager->startup();
+      conversationManager->startup();
 
       //ua.createSubscription("message-summary", uri, 120, Mime("application", "simple-message-summary")); // thread safe
 
@@ -1219,12 +1352,12 @@ ReConServerProcess::main (int argc, char** argv)
             {
 #ifdef WIN32
                input = _getch();
-               processKeyboard(input, *myConversationManager, ua);
+               processKeyboard(input, *conversationManager, ua);
 #else
                input = fgetc(stdin);
                fflush(stdin);
                //cout << "input: " << input << endl;
-               processKeyboard(input, *myConversationManager, ua);
+               processKeyboard(input, *conversationManager, ua);
 #endif
             }
          }
